/*        GENERADOR DE SUBTONOS 
 *      DISEÑADO POR ARMIN MORALES 
 *      CE3MRO - FF46pg
 *      
 *      Este programa permite generar 2 subtonos simultaneos para operar equipos con doble VFO o equipos monobanda para VHF y UHF
 *      
 *      https://www.radioaficion.pro
 *      Agosto 2019
 */


// Librerias

#include <EEPROM.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#include <TonePlayer.h>


// Definicion de Pines


#define TonePinA      (10)        // Especifica Pin de Salida de Tono A
#define TonePinB      (9)         // Especifica Pin de Salida de Tono B
#define pinA          (2)         // Pin Interrupcion para Encoder
#define pinB          (3)         // Pin Interrupcion para Encoder
#define buttonPin     (4)         // Pin Para el pulsador del Encoder


// Configura LCD 16x2

LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);         // Configura el Display LCD de 16x2 via I2C

// Variables

volatile byte aFlag = 0; // Avisa cuando esperamos un borde ascendente en el pinA para indicar que el codificador ha llegado a un tope
volatile byte bFlag = 0; // Avisa cuando esperamos un borde ascendente en el pinB para indicar que el codificador ha llegado a un tope (dirección opuesta a cuando se establece aFlag)
volatile byte encoderPos = 0; // Esta variable almacena nuestro valor actual de la posición del codificador. 
volatile byte oldEncPos = 0; // Almacena el último valor de posición del codificador para que podamos compararlo con la lectura actual y ver si ha cambiado.
volatile byte reading = 0; // Sirve para almacenar los valores directos que leemos de nuestros pines de interrupción antes de verificar si hemos movido un paso del encoder
byte oldButtonState = HIGH;  // Registra el Estado del pulsador del encoder.
const unsigned long debounceTime = 10;  // Define tiempo de refresco de las funciones en millisegundos
unsigned long buttonPressTime;  // Registra cuando el pulsador cambio por ultima vez
byte setting1 = 0;  // Esta variable configura el Tono A seleccionado.
byte setting2 = 0;  // Esta variable configura el Tono B seleccionado.
byte estadoboton = 0;  // Esta variable permite configurar el pulsador como interruptor 
byte estadoAnterior = 0;  // Esta variable permite configurar el pulsador como interruptor 

// Configura Libreria Toneplayer para generar el Tono B

TonePlayer tone1 (TCCR1A, TCCR1B, OCR1AH, OCR1AL, TCNT1H, TCNT1L);  // La salida del tono es por el pin D9.


// Configuracion de Arranque

void setup() {

  pinMode(pinA, INPUT_PULLUP);                  // Configura pinA del Encoder
  pinMode(pinB, INPUT_PULLUP);                  // Configura pinB del Encoder
  attachInterrupt(0, PinA, RISING);             // Configura Interrupcion para el PinA
  attachInterrupt(1, PinB, RISING);             // Configura Interrupcion para el PinB
  pinMode(buttonPin, INPUT_PULLUP);             // Configura Pin del Pulsador del Encoder
  pinMode(TonePinA, OUTPUT);                    // Configura Pin de Salida del Tono A
  pinMode(TonePinB, OUTPUT);                    // Configura Pin de Salida del Tono B      
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("Generador  CTCSS");                // Texto Bienvenida LCD Linea 1
  lcd.setCursor(0, 1);
  lcd.print("radioaficion.pro");                // Texto Bienvenida LCD Linea 2
  delay(4000);                                  // Retardo Texto Inicio en milisegundos.
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("A:");                              // Muestra Letra A en el Display
  lcd.setCursor(1, 1);
  lcd.print("B:");                              // Muestra Letra B en el Display



 // Configura Ultimo Estado
  
  estadoboton = EEPROM.read(2);                 // Lee la memoria EEPROM y Devuelve Ultima Posicion del Pulsador
  encoderPos = EEPROM.read(3);                  // Lee la memoria EEPROM y Devuelve Ultima Posicion del Encoder
  setting1 = EEPROM.read(0);                    // Lee la memoria EEPROM y Devuelve Ultima Posicion del Tono A
  setting2 = EEPROM.read(1);                    // Lee la memoria EEPROM y Devuelve Ultima Posicion del Tono B


  // Esta sentencia configura las posiciones iniciales del Tono A:
  
        if (setting1 == 0) {
        lcd.setCursor(3, 0);
        lcd.print(" Tono Apagado");
        noTone(TonePinA);
      }
      if (setting1 == 1) {
        lcd.setCursor(3, 0);
        lcd.print(" 67.0Hz PL:XZ");
        tone(TonePinA, 67.0);
      }
      if (setting1 == 2) {
        lcd.setCursor(3, 0);
        lcd.print(" 69.3Hz PL:WZ");
        tone(TonePinA, 69.3);
      }
      if (setting1 == 3) {
        lcd.setCursor(3, 0);
        lcd.print(" 71.9Hz PL:XA");
        tone(TonePinA, 71.9);
      }
      if (setting1 == 4) {
        lcd.setCursor(3, 0);
        lcd.print(" 74.4Hz PL:WA");
        tone(TonePinA, 74.4);
      }
      if (setting1 == 5) {
        lcd.setCursor(3, 0);
        lcd.print(" 77.0Hz PL:XB");
        tone(TonePinA, 77.0);
      }
      if (setting1 == 6) {
        lcd.setCursor(3, 0);
        lcd.print(" 79.7Hz PL:WB");
        tone(TonePinA, 79.7);
      }
      if (setting1 == 7) {
        lcd.setCursor(3, 0);
        lcd.print(" 82.5Hz PL:YZ");
        tone(TonePinA, 82.5);
      }
      if (setting1 == 8) {
        lcd.setCursor(3, 0);
        lcd.print(" 85.4Hz PL:YA");
        tone(TonePinA, 85.4);
      }
      if (setting1 == 9) {
        lcd.setCursor(3, 0);
        lcd.print(" 88.5Hz PL:YB");
        tone(TonePinA, 88.5);
      }
      if (setting1 == 10) {
        lcd.setCursor(3, 0);
        lcd.print(" 91.5Hz PL:ZZ");
        tone(TonePinA, 91.5);
      }
      if (setting1 == 11) {
        lcd.setCursor(3, 0);
        lcd.print(" 94.8Hz PL:ZA");
        tone(TonePinA, 94.8);
      }
      if (setting1 == 12) {
        lcd.setCursor(3, 0);
        lcd.print(" 97.4Hz PL:ZB");
        tone(TonePinA, 97.4);
      }
      if (setting1 == 13) {
        lcd.setCursor(3, 0);
        lcd.print("100.0Hz PL:1Z");
        tone(TonePinA, 100.0);
      }
      if (setting1 == 14) {
        lcd.setCursor(3, 0);
        lcd.print("103.5Hz PL:1A");
        tone(TonePinA, 103.5);
      }
      if (setting1 == 15) {
        lcd.setCursor(3, 0);
        lcd.print("107.2Hz PL:1B");
        tone(TonePinA, 107.2);
      }
      if (setting1 == 16) {
        lcd.setCursor(3, 0);
        lcd.print("110.9Hz PL:2Z");
        tone(TonePinA, 110.9);
      }
      if (setting1 == 17) {
        lcd.setCursor(3, 0);
        lcd.print("114.8Hz PL:2A");
        tone(TonePinA, 114.8);
      }
      if (setting1 == 18) {
        lcd.setCursor(3, 0);
        lcd.print("118.8Hz PL:2B");
        tone(TonePinA, 118.8);
      }
      if (setting1 == 19) {
        lcd.setCursor(3, 0);
        lcd.print("123.0Hz PL:3Z");
        tone(TonePinA, 123.0);
      }
      if (setting1 == 20) {
        lcd.setCursor(3, 0);
        lcd.print("127.3Hz PL:3A");
        tone(TonePinA, 127.3);
      }
      if (setting1 == 21) {
        lcd.setCursor(3, 0);
        lcd.print("131.8Hz PL:3B");
        tone(TonePinA, 131.8);
      }
      if (setting1 == 22) {
        lcd.setCursor(3, 0);
        lcd.print("136.5Hz PL:4Z");
        tone(TonePinA, 136.5);
      }
      if (setting1 == 23) {
        lcd.setCursor(3, 0);
        lcd.print("141.3Hz PL:4A");
        tone(TonePinA, 141.3);
      }
      if (setting1 == 24) {
        lcd.setCursor(3, 0);
        lcd.print("146.2Hz PL:4B");
        tone(TonePinA, 146.2);
      }
      if (setting1 == 25) {
        lcd.setCursor(3, 0);
        lcd.print("151.4Hz PL:5Z");
        tone(TonePinA, 151.4);
      }
      if (setting1 == 26) {
        lcd.setCursor(3, 0);
        lcd.print("156.7Hz PL:5A");
        tone(TonePinA, 156.7);
      }
      if (setting1 == 27) {
        lcd.setCursor(3, 0);
        lcd.print("162.2Hz PL:5B");
        tone(TonePinA, 162.2);
      }
      if (setting1 == 28) {
        lcd.setCursor(3, 0);
        lcd.print("167.9Hz PL:6Z");
        tone(TonePinA, 167.9);
      }
      if (setting1 == 29) {
        lcd.setCursor(3, 0);
        lcd.print("173.8Hz PL:6A");
        tone(TonePinA, 173.8);
      }
      if (setting1 == 30) {
        lcd.setCursor(3, 0);
        lcd.print("179.9Hz PL:6B");
        tone(TonePinA, 179.9);
      }
      if (setting1 == 31) {
        lcd.setCursor(3, 0);
        lcd.print("186.2Hz PL:7Z");
        tone(TonePinA, 186.2);
      }
      if (setting1 == 32) {
        lcd.setCursor(3, 0);
        lcd.print("192.8Hz PL:7A");
        tone(TonePinA, 192.8);
      }
      if (setting1 == 33) {
        lcd.setCursor(3, 0);
        lcd.print("203.5Hz PL:M1");
        tone(TonePinA, 203.5);
      }
      if (setting1 == 34) {
        lcd.setCursor(3, 0);
        lcd.print("206.5Hz PL:8Z");
        tone(TonePinA, 206.5);
      }
      if (setting1 == 35) {
        lcd.setCursor(3, 0);
        lcd.print("210.7Hz PL:M2");
        tone(TonePinA, 210.7);
      }
      if (setting1 == 36) {
        lcd.setCursor(3, 0);
        lcd.print("218.1Hz PL:M3");
        tone(TonePinA, 218.1);
      }
      if (setting1 == 37) {
        lcd.setCursor(3, 0);
        lcd.print("225.7Hz PL:M4");
        tone(TonePinA, 225.7);
      }
      if (setting1 == 38) {
        lcd.setCursor(3, 0);
        lcd.print("229.1Hz PL:9Z");
        tone(TonePinA, 229.1);
      }
      if (setting1 == 39) {
        lcd.setCursor(3, 0);
        lcd.print("233.6Hz PL:M5");
        tone(TonePinA, 233.6);
      }
      if (setting1 == 40) {
        lcd.setCursor(3, 0);
        lcd.print("241.8Hz PL:M6");
        tone(TonePinA, 241.8);
      }
      if (setting1 == 41) {
        lcd.setCursor(3, 0);
        lcd.print("250.3Hz PL:M7");
        tone(TonePinA, 250.3);
      }
      if (setting1 == 42) {
        lcd.setCursor(3, 0);
        lcd.print("254.1Hz PL:0Z");
        tone(TonePinA, 254.1);
      }


    // Esta sentencia configura las posiciones iniciales del Tono B:

      
      if (setting2 == 0) {
        lcd.setCursor(3, 1);
        lcd.print(" Tono Apagado");
        tone1.noTone();
      }
      if (setting2 == 1) {
        lcd.setCursor(3, 1);
        lcd.print(" 67.0Hz PL:XZ");
        tone1.tone(67.0);
      }
      if (setting2 == 2) {
        lcd.setCursor(3, 1);
        lcd.print(" 69.3Hz PL:WZ");
        tone1.tone(69.3);
      }
      if (setting2 == 3) {
        lcd.setCursor(3, 1);
        lcd.print(" 71.9Hz PL:XA");
        tone1.tone(71.9);
      }
      if (setting2 == 4) {
        lcd.setCursor(3, 1);
        lcd.print(" 74.4Hz PL:WA");
        tone1.tone(74.4);
      }
      if (setting2 == 5) {
        lcd.setCursor(3, 1);
        lcd.print(" 77.0Hz PL:XB");
        tone1.tone(77.0);
      }
      if (setting2 == 6) {
        lcd.setCursor(3, 1);
        lcd.print(" 79.7Hz PL:WB");
        tone1.tone(79.7);
      }
      if (setting2 == 7) {
        lcd.setCursor(3, 1);
        lcd.print(" 82.5Hz PL:YZ");
        tone1.tone(82.5);
      }
      if (setting2 == 8) {
        lcd.setCursor(3, 1);
        lcd.print(" 85.4Hz PL:YA");
       tone1.tone(85.4);
      }
      if (setting2 == 9) {
        lcd.setCursor(3, 1);
        lcd.print(" 88.5Hz PL:YB");
        tone1.tone(88.5);
      }
      if (setting2 == 10) {
        lcd.setCursor(3, 1);
        lcd.print(" 91.5Hz PL:ZZ");
        tone1.tone(91.5);
      }
      if (setting2 == 11) {
        lcd.setCursor(3, 1);
        lcd.print(" 94.8Hz PL:ZA");
        tone1.tone(94.8);
      }
      if (setting2 == 12) {
        lcd.setCursor(3, 1);
        lcd.print(" 97.4Hz PL:ZB");
       tone1.tone(97.4);
      }
      if (setting2 == 13) {
        lcd.setCursor(3, 1);
        lcd.print("100.0Hz PL:1Z");
        tone1.tone(100.0);
      }
      if (setting2 == 14) {
        lcd.setCursor(3, 1);
        lcd.print("103.5Hz PL:1A");
       tone1.tone( 103.5);
      }
      if (setting2 == 15) {
        lcd.setCursor(3, 1);
        lcd.print("107.2Hz PL:1B");
        tone1.tone(107.2);
      }
      if (setting2 == 16) {
        lcd.setCursor(3, 1);
        lcd.print("110.9Hz PL:2Z");
       tone1.tone(110.9);
      }
      if (setting2 == 17) {
        lcd.setCursor(3, 1);
        lcd.print("114.8Hz PL:2A");
       tone1.tone( 114.8);
      }
      if (setting2 == 18) {
        lcd.setCursor(3, 1);
        lcd.print("118.8Hz PL:2B");
        tone1.tone(118.8);
      }
      if (setting2 == 19) {
        lcd.setCursor(3, 1);
        lcd.print("123.0Hz PL:3Z");
      tone1.tone(123.0);
      }
      if (setting2 == 20) {
        lcd.setCursor(3, 1);
        lcd.print("127.3Hz PL:3A");
        tone1.tone(127.3);
      }
      if (setting2 == 21) {
        lcd.setCursor(3, 1);
        lcd.print("131.8Hz PL:3B");
        tone1.tone(131.8);
      }
      if (setting2 == 22) {
        lcd.setCursor(3, 1);
        lcd.print("136.5Hz PL:4Z");
        tone1.tone(136.5);
      }
      if (setting2 == 23) {
        lcd.setCursor(3, 1);
        lcd.print("141.3Hz PL:4A");
        tone1.tone(141.3);
      }
      if (setting2 == 24) {
        lcd.setCursor(3, 1);
        lcd.print("146.2Hz PL:4B");
        tone1.tone(146.2);
      }
      if (setting2 == 25) {
        lcd.setCursor(3, 1);
        lcd.print("151.4Hz PL:5Z");
        tone1.tone(151.4);
      }
      if (setting2 == 26) {
        lcd.setCursor(3, 1);
        lcd.print("156.7Hz PL:5A");
        tone1.tone(156.7);
      }
      if (setting2 == 27) {
        lcd.setCursor(3, 1);
        lcd.print("162.2Hz PL:5B");
        tone1.tone(162.2);
      }
      if (setting2 == 28) {
        lcd.setCursor(3, 1);
        lcd.print("167.9Hz PL:6Z");
        tone1.tone(167.9);
      }
      if (setting2 == 29) {
        lcd.setCursor(3, 1);
        lcd.print("173.8Hz PL:6A");
        tone1.tone(173.8);
      }
      if (setting2 == 30) {
        lcd.setCursor(3, 1);
        lcd.print("179.9Hz PL:6B");
        tone1.tone(179.9);
      }
      if (setting2 == 31) {
        lcd.setCursor(3, 1);
        lcd.print("186.2Hz PL:7Z");
        tone1.tone(186.2);
      }
      if (setting2 == 32) {
        lcd.setCursor(3, 1);
        lcd.print("192.8Hz PL:7A");
        tone1.tone(192.8);
      }
      if (setting2 == 33) {
        lcd.setCursor(3, 1);
        lcd.print("203.5Hz PL:M1");
        tone1.tone(203.5);
      }
      if (setting2 == 34) {
        lcd.setCursor(3, 1);
        lcd.print("206.5Hz PL:8Z");
        tone1.tone(206.5);
      }
      if (setting2 == 35) {
        lcd.setCursor(3, 1);
        lcd.print("210.7Hz PL:M2");
        tone1.tone(210.7);
      }
      if (setting2 == 36) {
        lcd.setCursor(3, 1);
        lcd.print("218.1Hz PL:M3");
        tone1.tone(218.1);
      }
      if (setting2 == 37) {
        lcd.setCursor(3, 1);
        lcd.print("225.7Hz PL:M4");
        tone1.tone(225.7);
      }
      if (setting2 == 38) {
        lcd.setCursor(3, 1);
        lcd.print("229.1Hz PL:9Z");
        tone1.tone(229.1);
      }
      if (setting2 == 39) {
        lcd.setCursor(3, 1);
        lcd.print("233.6Hz PL:M5");
        tone1.tone(233.6);
      }
      if (setting2 == 40) {
        lcd.setCursor(3, 1);
        lcd.print("241.8Hz PL:M6");
        tone1.tone(241.8);
      }
      if (setting2 == 41) {
        lcd.setCursor(3, 1);
        lcd.print("250.3Hz PL:M7");
        tone1.tone(250.3);
      }
      if (setting2 == 42) {
        lcd.setCursor(3, 1);
        lcd.print("254.1Hz PL:0Z");
        tone1.tone(254.1);
      }
}

// Termino de la configuracion de Inicio



void loop() {
  rotaryMenu();
}

void rotaryMenu() {

  // Esta sentencia devuelve a la posicion inicial al alcanzar el final.

  if (oldEncPos != encoderPos) {
    oldEncPos = encoderPos;
    if (encoderPos == 43) {
      encoderPos = 0;
    }
    if (encoderPos == 255) {
      encoderPos = 42;
    }


    // Esta sentencia configura el Tono A si se cambia la posicion del encoder

    if (estadoboton == 1) {
      setting1 = encoderPos;
      EEPROM.write(0, setting1);        // Guarda en la memoria EEPROM el tono A seleccionado
      EEPROM.write(3, encoderPos);      // Guarda en la memoria EEPROM la posicion del encoder
      
      if (setting1 == 0) {
        lcd.setCursor(3, 0);
        lcd.print(" Tono Apagado");
        noTone(TonePinA);
      }
      if (setting1 == 1) {
        lcd.setCursor(3, 0);
        lcd.print(" 67.0Hz PL:XZ");
        tone(TonePinA, 67.0);
      }
      if (setting1 == 2) {
        lcd.setCursor(3, 0);
        lcd.print(" 69.3Hz PL:WZ");
        tone(TonePinA, 69.3);
      }
      if (setting1 == 3) {
        lcd.setCursor(3, 0);
        lcd.print(" 71.9Hz PL:XA");
        tone(TonePinA, 71.9);
      }
      if (setting1 == 4) {
        lcd.setCursor(3, 0);
        lcd.print(" 74.4Hz PL:WA");
        tone(TonePinA, 74.4);
      }
      if (setting1 == 5) {
        lcd.setCursor(3, 0);
        lcd.print(" 77.0Hz PL:XB");
        tone(TonePinA, 77.0);
      }
      if (setting1 == 6) {
        lcd.setCursor(3, 0);
        lcd.print(" 79.7Hz PL:WB");
        tone(TonePinA, 79.7);
      }
      if (setting1 == 7) {
        lcd.setCursor(3, 0);
        lcd.print(" 82.5Hz PL:YZ");
        tone(TonePinA, 82.5);
      }
      if (setting1 == 8) {
        lcd.setCursor(3, 0);
        lcd.print(" 85.4Hz PL:YA");
        tone(TonePinA, 85.4);
      }
      if (setting1 == 9) {
        lcd.setCursor(3, 0);
        lcd.print(" 88.5Hz PL:YB");
        tone(TonePinA, 88.5);
      }
      if (setting1 == 10) {
        lcd.setCursor(3, 0);
        lcd.print(" 91.5Hz PL:ZZ");
        tone(TonePinA, 91.5);
      }
      if (setting1 == 11) {
        lcd.setCursor(3, 0);
        lcd.print(" 94.8Hz PL:ZA");
        tone(TonePinA, 94.8);
      }
      if (setting1 == 12) {
        lcd.setCursor(3, 0);
        lcd.print(" 97.4Hz PL:ZB");
        tone(TonePinA, 97.4);
      }
      if (setting1 == 13) {
        lcd.setCursor(3, 0);
        lcd.print("100.0Hz PL:1Z");
        tone(TonePinA, 100.0);
      }
      if (setting1 == 14) {
        lcd.setCursor(3, 0);
        lcd.print("103.5Hz PL:1A");
        tone(TonePinA, 103.5);
      }
      if (setting1 == 15) {
        lcd.setCursor(3, 0);
        lcd.print("107.2Hz PL:1B");
        tone(TonePinA, 107.2);
      }
      if (setting1 == 16) {
        lcd.setCursor(3, 0);
        lcd.print("110.9Hz PL:2Z");
        tone(TonePinA, 110.9);
      }
      if (setting1 == 17) {
        lcd.setCursor(3, 0);
        lcd.print("114.8Hz PL:2A");
        tone(TonePinA, 114.8);
      }
      if (setting1 == 18) {
        lcd.setCursor(3, 0);
        lcd.print("118.8Hz PL:2B");
        tone(TonePinA, 118.8);
      }
      if (setting1 == 19) {
        lcd.setCursor(3, 0);
        lcd.print("123.0Hz PL:3Z");
        tone(TonePinA, 123.0);
      }
      if (setting1 == 20) {
        lcd.setCursor(3, 0);
        lcd.print("127.3Hz PL:3A");
        tone(TonePinA, 127.3);
      }
      if (setting1 == 21) {
        lcd.setCursor(3, 0);
        lcd.print("131.8Hz PL:3B");
        tone(TonePinA, 131.8);
      }
      if (setting1 == 22) {
        lcd.setCursor(3, 0);
        lcd.print("136.5Hz PL:4Z");
        tone(TonePinA, 136.5);
      }
      if (setting1 == 23) {
        lcd.setCursor(3, 0);
        lcd.print("141.3Hz PL:4A");
        tone(TonePinA, 141.3);
      }
      if (setting1 == 24) {
        lcd.setCursor(3, 0);
        lcd.print("146.2Hz PL:4B");
        tone(TonePinA, 146.2);
      }
      if (setting1 == 25) {
        lcd.setCursor(3, 0);
        lcd.print("151.4Hz PL:5Z");
        tone(TonePinA, 151.4);
      }
      if (setting1 == 26) {
        lcd.setCursor(3, 0);
        lcd.print("156.7Hz PL:5A");
        tone(TonePinA, 156.7);
      }
      if (setting1 == 27) {
        lcd.setCursor(3, 0);
        lcd.print("162.2Hz PL:5B");
        tone(TonePinA, 162.2);
      }
      if (setting1 == 28) {
        lcd.setCursor(3, 0);
        lcd.print("167.9Hz PL:6Z");
        tone(TonePinA, 167.9);
      }
      if (setting1 == 29) {
        lcd.setCursor(3, 0);
        lcd.print("173.8Hz PL:6A");
        tone(TonePinA, 173.8);
      }
      if (setting1 == 30) {
        lcd.setCursor(3, 0);
        lcd.print("179.9Hz PL:6B");
        tone(TonePinA, 179.9);
      }
      if (setting1 == 31) {
        lcd.setCursor(3, 0);
        lcd.print("186.2Hz PL:7Z");
        tone(TonePinA, 186.2);
      }
      if (setting1 == 32) {
        lcd.setCursor(3, 0);
        lcd.print("192.8Hz PL:7A");
        tone(TonePinA, 192.8);
      }
      if (setting1 == 33) {
        lcd.setCursor(3, 0);
        lcd.print("203.5Hz PL:M1");
        tone(TonePinA, 203.5);
      }
      if (setting1 == 34) {
        lcd.setCursor(3, 0);
        lcd.print("206.5Hz PL:8Z");
        tone(TonePinA, 206.5);
      }
      if (setting1 == 35) {
        lcd.setCursor(3, 0);
        lcd.print("210.7Hz PL:M2");
        tone(TonePinA, 210.7);
      }
      if (setting1 == 36) {
        lcd.setCursor(3, 0);
        lcd.print("218.1Hz PL:M3");
        tone(TonePinA, 218.1);
      }
      if (setting1 == 37) {
        lcd.setCursor(3, 0);
        lcd.print("225.7Hz PL:M4");
        tone(TonePinA, 225.7);
      }
      if (setting1 == 38) {
        lcd.setCursor(3, 0);
        lcd.print("229.1Hz PL:9Z");
        tone(TonePinA, 229.1);
      }
      if (setting1 == 39) {
        lcd.setCursor(3, 0);
        lcd.print("233.6Hz PL:M5");
        tone(TonePinA, 233.6);
      }
      if (setting1 == 40) {
        lcd.setCursor(3, 0);
        lcd.print("241.8Hz PL:M6");
        tone(TonePinA, 241.8);
      }
      if (setting1 == 41) {
        lcd.setCursor(3, 0);
        lcd.print("250.3Hz PL:M7");
        tone(TonePinA, 250.3);
      }
      if (setting1 == 42) {
        lcd.setCursor(3, 0);
        lcd.print("254.1Hz PL:0Z");
        tone(TonePinA, 254.1);
      }
    }



   // Esta sentencia configura el Tono B si se cambia la posicion del encoder


    if (estadoboton == 0) {
      setting2 = encoderPos;
      EEPROM.write(1, setting2);         // Guarda en la memoria EEPROM el tono B seleccionado
      EEPROM.write(3, encoderPos);       // Guarda en la memoria EEPROM la posicion del encoder

      if (setting2 == 0) {
        lcd.setCursor(3, 1);
        lcd.print(" Tono Apagado");
        tone1.noTone();
      }
      if (setting2 == 1) {
        lcd.setCursor(3, 1);
        lcd.print(" 67.0Hz PL:XZ");
        tone1.tone(67.0);
      }
      if (setting2 == 2) {
        lcd.setCursor(3, 1);
        lcd.print(" 69.3Hz PL:WZ");
        tone1.tone(69.3);
      }
      if (setting2 == 3) {
        lcd.setCursor(3, 1);
        lcd.print(" 71.9Hz PL:XA");
        tone1.tone(71.9);
      }
      if (setting2 == 4) {
        lcd.setCursor(3, 1);
        lcd.print(" 74.4Hz PL:WA");
        tone1.tone(74.4);
      }
      if (setting2 == 5) {
        lcd.setCursor(3, 1);
        lcd.print(" 77.0Hz PL:XB");
        tone1.tone(77.0);
      }
      if (setting2 == 6) {
        lcd.setCursor(3, 1);
        lcd.print(" 79.7Hz PL:WB");
        tone1.tone(79.7);
      }
      if (setting2 == 7) {
        lcd.setCursor(3, 1);
        lcd.print(" 82.5Hz PL:YZ");
        tone1.tone(82.5);
      }
      if (setting2 == 8) {
        lcd.setCursor(3, 1);
        lcd.print(" 85.4Hz PL:YA");
       tone1.tone(85.4);
      }
      if (setting2 == 9) {
        lcd.setCursor(3, 1);
        lcd.print(" 88.5Hz PL:YB");
        tone1.tone(88.5);
      }
      if (setting2 == 10) {
        lcd.setCursor(3, 1);
        lcd.print(" 91.5Hz PL:ZZ");
        tone1.tone(91.5);
      }
      if (setting2 == 11) {
        lcd.setCursor(3, 1);
        lcd.print(" 94.8Hz PL:ZA");
        tone1.tone(94.8);
      }
      if (setting2 == 12) {
        lcd.setCursor(3, 1);
        lcd.print(" 97.4Hz PL:ZB");
       tone1.tone(97.4);
      }
      if (setting2 == 13) {
        lcd.setCursor(3, 1);
        lcd.print("100.0Hz PL:1Z");
        tone1.tone(100.0);
      }
      if (setting2 == 14) {
        lcd.setCursor(3, 1);
        lcd.print("103.5Hz PL:1A");
       tone1.tone( 103.5);
      }
      if (setting2 == 15) {
        lcd.setCursor(3, 1);
        lcd.print("107.2Hz PL:1B");
        tone1.tone(107.2);
      }
      if (setting2 == 16) {
        lcd.setCursor(3, 1);
        lcd.print("110.9Hz PL:2Z");
       tone1.tone(110.9);
      }
      if (setting2 == 17) {
        lcd.setCursor(3, 1);
        lcd.print("114.8Hz PL:2A");
       tone1.tone( 114.8);
      }
      if (setting2 == 18) {
        lcd.setCursor(3, 1);
        lcd.print("118.8Hz PL:2B");
        tone1.tone(118.8);
      }
      if (setting2 == 19) {
        lcd.setCursor(3, 1);
        lcd.print("123.0Hz PL:3Z");
      tone1.tone(123.0);
      }
      if (setting2 == 20) {
        lcd.setCursor(3, 1);
        lcd.print("127.3Hz PL:3A");
        tone1.tone(127.3);
      }
      if (setting2 == 21) {
        lcd.setCursor(3, 1);
        lcd.print("131.8Hz PL:3B");
        tone1.tone(131.8);
      }
      if (setting2 == 22) {
        lcd.setCursor(3, 1);
        lcd.print("136.5Hz PL:4Z");
        tone1.tone(136.5);
      }
      if (setting2 == 23) {
        lcd.setCursor(3, 1);
        lcd.print("141.3Hz PL:4A");
        tone1.tone(141.3);
      }
      if (setting2 == 24) {
        lcd.setCursor(3, 1);
        lcd.print("146.2Hz PL:4B");
        tone1.tone(146.2);
      }
      if (setting2 == 25) {
        lcd.setCursor(3, 1);
        lcd.print("151.4Hz PL:5Z");
        tone1.tone(151.4);
      }
      if (setting2 == 26) {
        lcd.setCursor(3, 1);
        lcd.print("156.7Hz PL:5A");
        tone1.tone(156.7);
      }
      if (setting2 == 27) {
        lcd.setCursor(3, 1);
        lcd.print("162.2Hz PL:5B");
        tone1.tone(162.2);
      }
      if (setting2 == 28) {
        lcd.setCursor(3, 1);
        lcd.print("167.9Hz PL:6Z");
        tone1.tone(167.9);
      }
      if (setting2 == 29) {
        lcd.setCursor(3, 1);
        lcd.print("173.8Hz PL:6A");
        tone1.tone(173.8);
      }
      if (setting2 == 30) {
        lcd.setCursor(3, 1);
        lcd.print("179.9Hz PL:6B");
        tone1.tone(179.9);
      }
      if (setting2 == 31) {
        lcd.setCursor(3, 1);
        lcd.print("186.2Hz PL:7Z");
        tone1.tone(186.2);
      }
      if (setting2 == 32) {
        lcd.setCursor(3, 1);
        lcd.print("192.8Hz PL:7A");
        tone1.tone(192.8);
      }
      if (setting2 == 33) {
        lcd.setCursor(3, 1);
        lcd.print("203.5Hz PL:M1");
        tone1.tone(203.5);
      }
      if (setting2 == 34) {
        lcd.setCursor(3, 1);
        lcd.print("206.5Hz PL:8Z");
        tone1.tone(206.5);
      }
      if (setting2 == 35) {
        lcd.setCursor(3, 1);
        lcd.print("210.7Hz PL:M2");
        tone1.tone(210.7);
      }
      if (setting2 == 36) {
        lcd.setCursor(3, 1);
        lcd.print("218.1Hz PL:M3");
        tone1.tone(218.1);
      }
      if (setting2 == 37) {
        lcd.setCursor(3, 1);
        lcd.print("225.7Hz PL:M4");
        tone1.tone(225.7);
      }
      if (setting2 == 38) {
        lcd.setCursor(3, 1);
        lcd.print("229.1Hz PL:9Z");
        tone1.tone(229.1);
      }
      if (setting2 == 39) {
        lcd.setCursor(3, 1);
        lcd.print("233.6Hz PL:M5");
        tone1.tone(233.6);
      }
      if (setting2 == 40) {
        lcd.setCursor(3, 1);
        lcd.print("241.8Hz PL:M6");
        tone1.tone(241.8);
      }
      if (setting2 == 41) {
        lcd.setCursor(3, 1);
        lcd.print("250.3Hz PL:M7");
        tone1.tone(250.3);
      }
      if (setting2 == 42) {
        lcd.setCursor(3, 1);
        lcd.print("254.1Hz PL:0Z");
        tone1.tone(254.1);
      }
    }
  }


   // Detecta Pulsacion del Encoder y lo configura como Interruptor
   
  byte buttonState = digitalRead (buttonPin);
  if (buttonState != oldButtonState) {
    if (millis () - buttonPressTime >= debounceTime) {
      buttonPressTime = millis ();
      if ((buttonState == LOW) && (estadoAnterior == 0)) {
        estadoboton = 1 - estadoboton;
        delay(200);
        if (estadoboton==1){
          encoderPos=setting1;
        }
        if (estadoboton==0){
          encoderPos=setting2;
        }
        EEPROM.write(2, estadoboton);                        // Guarda en la memoria EEPROM el estado del pulsador como interruptor
      }
      estadoAnterior = buttonState;  
    }
  }



    //  Muestra y Configura Posicion de Seleccion en LCD


// Muestra en el LCD cuando se selecciona Tono A:

  if (estadoboton == 1) {
    setting1 = encoderPos;
    lcd.setCursor(0, 0);
    lcd.print(">");
    lcd.setCursor(0, 1);
    lcd.print(" ");
  } 

// Muestra en el LCD cuando se selecciona Tono B:

  if (estadoboton == 0) {
    setting2 = encoderPos;
    lcd.setCursor(0, 0);
    lcd.print(" ");
    lcd.setCursor(0, 1);
    lcd.print(">");
  }
}

// Sentencia para detectar el movimiento del encoder en ambos sentidos utilizando las interrupcciones.

void PinA() {
  cli();                                           // detiene las interrupciones que suceden antes de leer los valores de pin A
  reading = PIND & 0xC;                            // lee los ocho valores de pines y luego elimina todos los valores excepto los de pinA y pinB
  if (reading == B00001100 && aFlag) {             // Verifica que tengamos los dos pines en la posicion (ALTO) y que estemos en espera del borde ascendente del PinA
    encoderPos --;                                 // Devolver el recuento de posición del encoder
    bFlag = 0;                                     // Restablece marcas para el próximo turno
    aFlag = 0;                                     // Restablece marcas para el próximo turno
  }
  else if (reading == B00000100) bFlag = 1;        // Señal de que esperamos que el pinA señale la transición a la siguiente posicion del encoder.
  sei();                                           // Reinicia las interrupcciones 
}

void PinB() {
  cli();                                           // detiene las interrupciones que suceden antes de leer los valores de pin B
  reading = PIND & 0xC;                            // lee los ocho valores de pines y luego elimina todos los valores excepto los de pinA y pinB
  if (reading == B00001100 && bFlag) {             // Verifica que tengamos los dos pines en la posicion (ALTO) y que estemos en espera del borde ascendente del PinB
    encoderPos ++;                                 // Incrementa el recuento de posición del encoder
    bFlag = 0;                                     // Restablece marcas para el próximo turno
    aFlag = 0;                                     // Restablece marcas para el próximo turno
  }
  else if (reading == B00001000) aFlag = 1;        // Señal de que esperamos que el pinB señale la transición a la siguiente posicion del encoder.
  sei();                                           // Reinicia las interrupcciones
}



// Fin del Programa
